/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage combobox history
//-----------------------------------------------------------------------------
#include "ComboboxHistory.h"
#include <windowsX.h>
#include <stdio.h>
#include "../../Ini/IniFile.h"

#define CComboboxHistory_MaxItemLength 1024 
#define CComboboxHistory_ItemBaseName _T("Item")

CComboboxHistory::CComboboxHistory(HWND ComboboxHandle,TCHAR* ConfigFileName,TCHAR* BaseKeyName)
{
    this->ComboboxHandle = ComboboxHandle;
    
    if (ConfigFileName)
        this->ConfigFileName = _tcsdup(ConfigFileName);
    else
        this->ConfigFileName = NULL;

    if (BaseKeyName)
        this->BaseKeyName = _tcsdup(BaseKeyName);
    else
        this->BaseKeyName = NULL;

    for (SIZE_T Cnt = 0; Cnt<CComboboxHistory_Length; Cnt++)
    {
        this->History[Cnt] = _tcsdup(_T(""));
    }
}

void CComboboxHistory::FreeMemory()
{
    for (SIZE_T Cnt = 0; Cnt<CComboboxHistory_Length; Cnt++)
    {
        if ( this->History[Cnt] )
        {
            free(this->History[Cnt]);
            this->History[Cnt] = 0;
        }
    }
}

CComboboxHistory::~CComboboxHistory()
{
    this->FreeMemory();
    if (this->ConfigFileName)
        free(this->ConfigFileName);
    if (this->BaseKeyName)
        free(this->BaseKeyName);
}

BOOL CComboboxHistory::AddCurrentContentToHistoryIfNeeded()
{
    BOOL bRet;
    SIZE_T Len = ::GetWindowTextLength(this->ComboboxHandle)+1;
    TCHAR* NewString = new TCHAR[Len];
    *NewString=0;
    ::GetWindowText(this->ComboboxHandle,NewString,(int)Len);

    bRet = this->AddContentToHistoryIfNeeded(NewString);

    delete[] NewString;

    return bRet;
}

BOOL CComboboxHistory::AddContentToHistoryIfNeeded(TCHAR* Content)
{
    SIZE_T Cnt;
    for (Cnt = 0; Cnt<CComboboxHistory_Length; Cnt++)
    {
        if (this->History[Cnt] == 0)
            continue;
        // if item already in history
        if ( _tcscmp(this->History[Cnt],Content) == 0)
        {
            // put item from position Cnt to position 0

            if (Cnt) // if Cnt == 0, data is the same as last one --> nothing to do
            {
                // save item pointer
                TCHAR* CurrentItem = this->History[Cnt];

                // put history item from index Cnt to index 0
                SIZE_T Cnt2;
                for (Cnt2 = Cnt; Cnt2 ; Cnt2--)
                {
                    this->History[Cnt2]=this->History[Cnt2-1];
                }

                // set item at pos 0
                this->History[0]=CurrentItem;
            }
            break;
        }
    }
    if (Cnt == CComboboxHistory_Length) // if not found
    {
        ///////////
        // add item
        ///////////

        // free last item
        free(this->History[CComboboxHistory_Length-1]);

        // next move items
        for (Cnt = CComboboxHistory_Length-1; Cnt; Cnt--)
        {
            this->History[Cnt]=this->History[Cnt-1];
        }

        // next add item at pos 0
        this->History[0]=_tcsdup(Content);
    }

    // update combo content
    this->FillComboboxWithHistory(TRUE);

    return TRUE;
}

void CComboboxHistory::FillComboboxWithHistory(BOOL AutoSelectLastItem)
{
    BOOL bAddToCombo;
    SIZE_T Cnt;
    ComboBox_ResetContent(this->ComboboxHandle);
    for (Cnt=0;Cnt<CComboboxHistory_Length;Cnt++)
    {
        if (this->History[Cnt]==0)
            continue;
        if (*this->History[Cnt])
            bAddToCombo = TRUE;
        else
        {
            // if not last element
            if (Cnt<(CComboboxHistory_Length-1))
            {
                // if next item is not empty
                bAddToCombo = (*this->History[Cnt+1]!=0);
            }
            else
                // history contains only empty strings --> no need to add to combo
                bAddToCombo = FALSE;
        }
        if (bAddToCombo)
            ComboBox_AddString(this->ComboboxHandle,this->History[Cnt]);
    }

    if (AutoSelectLastItem)
    {
        // set selected item
        ComboBox_SetCurSel(this->ComboboxHandle,0);
        // clear selection text of selected item
        ::PostMessage(this->ComboboxHandle,CB_SETEDITSEL,0,0);
    }
}



BOOL CComboboxHistory::LoadHistory(BOOL AutoSelectLastItem)
{
    
    TCHAR Tmp[CComboboxHistory_MaxItemLength];
    BOOL bRet;
    SIZE_T Cnt;
    TCHAR KeyName[MAX_PATH];

    if (!this->ConfigFileName)
        return FALSE;
    if (!this->BaseKeyName)
        return FALSE;

    bRet = TRUE;
    this->FreeMemory();
    CIniFile IniFile(this->ConfigFileName);
    IniFile.SetCurrentSectionName(this->BaseKeyName);
    for (Cnt = 0; Cnt<CComboboxHistory_Length; Cnt++)
    {
        _sntprintf(KeyName,_countof(KeyName), CComboboxHistory_ItemBaseName _T("_%u"),Cnt);
        *Tmp=0;
        bRet = bRet && IniFile.Get(KeyName,_T(""),Tmp,CComboboxHistory_MaxItemLength);
        this->History[Cnt]=_tcsdup(Tmp);
    } 

    this->FillComboboxWithHistory(AutoSelectLastItem);

    return bRet;
}

BOOL CComboboxHistory::SaveHistory()
{
    BOOL bRet;
    SIZE_T Cnt;
    TCHAR KeyName[MAX_PATH];

    if (!this->ConfigFileName)
        return FALSE;
    if (!this->BaseKeyName)
        return FALSE;

    bRet = TRUE;
    CIniFile IniFile(this->ConfigFileName);
    IniFile.SetCurrentSectionName(this->BaseKeyName);
    for (Cnt = 0; Cnt<CComboboxHistory_Length; Cnt++)
    {
        if (this->History[Cnt]==0)
            continue;
        _sntprintf(KeyName,_countof(KeyName), CComboboxHistory_ItemBaseName _T("_%u"),Cnt);
        bRet = bRet && IniFile.Set(KeyName,this->History[Cnt]);
    }  

    return bRet;
}